// **********************************************************************
// SCIDYNE   649 School Street   Pembroke, MA 02359-3649 USA
// Tel: (781) 293-3059   Fax: (781) 293-4034   URL: www.scidyne.com
// **********************************************************************
// Project:  ADIO-104 Demonstration Program
// File:     ADIO_EX1.CPP
// Revision: 0.01
// Date:     01-29-02
// Author:   Mark W. Durgin
// *************************************************************************
// This C program demonstrates the basic operation of the SCIDYNE ADIO-104
// data acquistion module (Model 100-7638). For clarity and ease in
// understanding the program has been written with simplicity taking
// precedence over robustness. It is supplied without any implied or
// expressed warranties or claims. The user accepts all consequences
// resulting from its use and has the sole responsibility in determining its
// suitability for any application.
//
// The program operates by first initializing important system variables and
// then runs in a continous loop until stopped by pressing the 'X' key.
// The analog inputs are digitized and displayed as raw numbers and voltages.
//
// The sixteen analog inputs are setup to measure voltages in the range of
// +/- 10Vdc using internally timed acquisition.  The setup parameters
// and resulting values are stored in an array of structures where each
// member structure corresponds to a separate analog input channel. For
// simplicity the DAS chip operations are polled but the code easily changed
// to use interrupts if desired.
//
// *************************************************************************
// History:
//   01-29-02  R0.01  Original Development Code
// *************************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <dos.h>

//--- Declare function prototypes
void 	program_initz(void);              // Performs program setups and intialization
void 	write_analog_output(int, int);    // Writes to an Analog Output
int 	read_analog_input(int);           // Reads an Analog Input

//--- Define program equates
#define	BASE_ADDR 		 	0x300						// Define where in IO map ADIO-104 is located
#define	TRUE					 	1								// Logic TRUE state
#define FALSE					 	!TRUE		 				// Logic FALSE state
#define AIMAX	          16	   					// Number of analog inputs
#define AOMAX	          8	     					// Number of analog outputs

//--- Define ADIO hardware equates
#define AOCH0_LB				BASE_ADDR+0    	// 12-bit analog out #0 LB
#define AOCH0_HB				BASE_ADDR+1    	// 12-bit analog out #0 HB
#define AOCH1_LB				BASE_ADDR+2    	// 12-bit analog out #1 LB
#define AOCH1_HB				BASE_ADDR+3    	// 12-bit analog out #1 HB
#define AOCH2_LB				BASE_ADDR+4    	// 12-bit analog out #2 LB
#define AOCH2_HB				BASE_ADDR+5    	// 12-bit analog out #2 HB
#define AOCH3_LB				BASE_ADDR+6    	// 12-bit analog out #3 LB
#define AOCH3_HB				BASE_ADDR+7    	// 12-bit analog out #3 HB
#define AOCH4_LB				BASE_ADDR+8    	// 12-bit analog out #4 LB
#define AOCH4_HB				BASE_ADDR+9    	// 12-bit analog out #4 HB
#define AOCH5_LB				BASE_ADDR+10   	// 12-bit analog out #5 LB
#define AOCH5_HB				BASE_ADDR+11   	// 12-bit analog out #5 HB
#define AOCH6_LB				BASE_ADDR+12   	// 12-bit analog out #6 LB
#define AOCH6_HB				BASE_ADDR+13   	// 12-bit analog out #6 HB
#define AOCH7_LB				BASE_ADDR+14   	// 12-bit analog out #7 LB
#define AOCH7_HB				BASE_ADDR+15   	// 12-bit analog out #7 HB
#define	AO_UPDATE		  	BASE_ADDR+16   	// Write to simultaneously update all analog outputs
#define SIM_DAS_CTRL  	BASE_ADDR+17   	// Simultaneous DAS control register
#define DAS0_CTRL 			BASE_ADDR+18   	// DAS0 control register
#define	DAS0_LB			  	BASE_ADDR+18   	// DAS0 12-bit Result LB
#define	DAS0_HB			  	BASE_ADDR+19   	// DAS0 12-bit result HB
#define DAS1_CTRL 			BASE_ADDR+20   	// DAS1 control register
#define	DAS1_LB			  	BASE_ADDR+20   	// DAS1 12-bit Result LB
#define	DAS1_HB			  	BASE_ADDR+21   	// DAS1 12-bit result HB
#define PORTA         	BASE_ADDR+22   	// Digital PORT-A
#define PORTB         	BASE_ADDR+23   	// Digital PORT-B
#define PORTC         	BASE_ADDR+24   	// Digital PORT-C
#define PACNT         	BASE_ADDR+25   	// 8-Bit Pulse Accumulator
#define INTR_STATUS 		BASE_ADDR+26   	// Intrrupt Status register
#define CONFIG        	BASE_ADDR+27   	// ADIO Configuration register
#define RSVD0         	BASE_ADDR+28   	// Reserved for future
#define RSVD1         	BASE_ADDR+29   	// Reserved for future
#define RSVD2         	BASE_ADDR+30   	// Reserved for future
#define RSVD3         	BASE_ADDR+31   	// Reserved for future

//--- MAX197 Control register bit positions and single bit masks
#define DAS_CTRL_BIP_BIT 		      3     // Conversion: 0 = Unipolar, 1 = Bipolar
#define DAS_CTRL_RNG_BIT 		      4     // Full Scale: 0 = 5, 1 = 10V
#define DAS_CTRL_ACQMOD_BIT	      5     // Acquistion: 0 = Internal, 1 = External
#define DAS_CTRL_PD0_BIT 		      6     // Clock and Power Down
#define DAS_CTRL_PD1_BIT 		      7     // Clock and Power Down

#define DAS_CTRL_BIP_MASK 		    (1 << DAS_CTRL_BIP_BIT)
#define DAS_CTRL_RNG_MASK 		    (1 << DAS_CTRL_RNG_BIT)
#define DAS_CTRL_ACQMOD_MASK	    (1 << DAS_CTRL_ACQMOD_BIT)
#define DAS_CTRL_PD0_MASK		      (1 << DAS_CTRL_PD0_BIT)
#define DAS_CTRL_PD1_MASK		      (1 << DAS_CTRL_PD1_BIT)

//--- Interrupt Status register bit positions and single bit masks
#define INTR_STATUS_DAS0_BIT 	    0   	// DAS0 Conversion Status:  		1 = New value ready
#define INTR_STATUS_DAS1_BIT 	    1   	// DAS1 Conversion Status:  		1 = New value ready
#define INTR_STATUS_EXT_BIT 	    2   	// EXTernal Interrupt Status:  	1 = High-to-Low transition occured
#define INTR_STATUS_PAOVF_BIT     3     // Counter Overflow:  					1 = Counter went 0xff to 0x00

#define INTR_STATUS_DAS0_MASK     (1 << INTR_STATUS_DAS0_BIT)
#define INTR_STATUS_DAS1_MASK     (1 << INTR_STATUS_DAS1_BIT)
#define INTR_STATUS_EXT_MASK      (1 << INTR_STATUS_EXT_BIT)
#define INTR_STATUS_PAOVF_MASK    (1 << INTR_STATUS_PAOVF_BIT)

//--- Config register bit positions and single bit masks
#define CONFIG_DIR_PA_LO_BIT 	    0   	// PORTA Bits 0-3 Direction control, 0 = Input, 1 = Output
#define CONFIG_DIR_PA_HI_BIT 	    1     // PORTA Bits 4-7 Direction control, 0 = Input, 1 = Output
#define CONFIG_DIR_PB_LO_BIT 	    2   	// PORTB Bits 0,1 Direction control, 0 = Input, 1 = Output
#define CONFIG_DIR_PB_HI_BIT 	    3   	// PORTB Bits 0-3 Direction control, 0 = Input, 1 = Output
#define CONFIG_DAS_CLK_SEL_BIT 	  4   	// DAS Clock scaler selection, 0 = /8 (default)  1 = /4
#define CONFIG_DAS_IEN_BIT 	      5   	// DAS Interrupt enable, 1 = Enable interrupt
#define CONFIG_EXT_IEN_BIT 	      6   	// EXTernal Interrupt enable, 1 = Enable interrupt
#define CONFIG_PAOVF_IEN_BIT      7   	// Pulse Accum. Overflow Interrupt enable, 1 = Enable interrupt

#define CONFIG_DIR_PA_LO_MASK     (1 << CONFIG_DIR_PA_LO_BIT)
#define CONFIG_DIR_PA_HI_MASK     (1 << CONFIG_DIR_PA_HI_BIT)
#define CONFIG_DIR_PB_LO_MASK     (1 << CONFIG_DIR_PB_LO_BIT)
#define CONFIG_DIR_PB_HI_MASK     (1 << CONFIG_DIR_PB_HI_BIT)
#define CONFIG_DAS_CLK_SEL_MASK   (1 << CONFIG_DAS_CLK_SEL_BIT)
#define CONFIG_DAS_IEN_MASK       (1 << CONFIG_DAS_IEN_BIT)
#define CONFIG_EXT_IEN_MASK       (1 << CONFIG_EXT_IEN_BIT)
#define CONFIG_PAOVF_IEN_MASK     (1 << CONFIG_PAOVF_IEN_BIT)

//--- Standard bit equates and masks
#define BIT_7               7
#define BIT_6               6
#define BIT_5               5
#define BIT_4               4
#define BIT_3               3
#define BIT_2               2
#define BIT_1               1
#define BIT_0               0
#define MASK_BIT_7          (1 << BIT_7)
#define MASK_BIT_6          (1 << BIT_6)
#define MASK_BIT_5          (1 << BIT_5)
#define MASK_BIT_4          (1 << BIT_4)
#define MASK_BIT_3          (1 << BIT_3)
#define MASK_BIT_2          (1 << BIT_2)
#define MASK_BIT_1          (1 << BIT_1)
#define MASK_BIT_0          (1 << BIT_0)


//--- Declare global variables and strucures
union  bytes_or_word{    								// Variables created using this union
	unsigned char 	byte[2];    					// can be accessed as either 2 bytes or
	int 		word;       									// as one word. Note: Byte[0] = LSB
};

struct { // Holds setups and results for corresponding analog inputs
	union  		bytes_or_word 	raw;    	  // Holds raw analog input result
  float                     voltage;    // Calculated voltage value
	unsigned 	char						das_ctrl;   // Holds the DAS channel control parameters
} ain_channel[AIMAX];

struct { // Holds data values for corresponding analog outputs
	union  		bytes_or_word 	wr_value;   // Holds outgoing analog output value
	union  		bytes_or_word 	rd_value;   // Holds read-back of analog output value.  Not used in
} aout_channel[AOMAX];                  //   this program but DAC read-back is supported by
																				//   ADIO-104 hardware.


//======================================================================================================
/***********************************************************************
   This is the main loop
***********************************************************************/
int main(void)
{
	//--- Declare local variables
	int keyval;   // Holds keyboard values
  int x;        // General purpose variable

	//--- Initz the program
  program_initz();  // Initz the overall program

  //--- This is the main loop
	while(TRUE)
	{
    //--- Check if any keyboard activity
		if(kbhit())
		{
      keyval = getch();   // Read the keyboard

      if(!keyval)         // Is it an extended code?
        keyval = getch(); // Yes, read next byte for actual key

			  switch (keyval)  	// Read the keystroke
			  {
          case 'X':  // Quit the program
          case 'x':
				  {
            // restore the original interrupt conditions
            clrscr();
            return(0);
            break;
				  }

          default:
					break;
		    } // end 'switch' for which keys pressed
    } // end "if" for any keys pressed


    //--- Digitize and display all the analog inputs
    gotoxy(1,4);																// Locate the cursor on user screen
    printf("CHANNEL     RAW     VOLTS\n"); 			// Print a header

    for(x = 0; x < AIMAX; x++)
    {
      read_analog_input(x);
      printf("AICH-%02d:   % 05d   % 2.6f\n",x, ain_channel[x].raw.word, ain_channel[x].voltage);
    }


	} // end while
} // end of main



/***********************************************************************
    FUNCTION: read_analog_input
 DESCRIPTION: Reads an analog input. Result is stored in corresponding
 						  ain_channel structure as well as being returned. The voltage
              value is calulated based on the the analog input
              RNG and BIP configuration bits.

       ENTER: channel_number 	: Is the analog input to digitize.

      RETURN: Signed integer raw value of designated analog input.
************************************************************************/
int read_analog_input(int channel_number)
{
    //--- Determine which DAS chip to use

    if(channel_number < AIMAX/2)
    { // Analog inputs [0:7] are on DAS0

			outp(DAS0_CTRL,ain_channel[channel_number].das_ctrl); 			// Start a convesion on the designated channel

    	while(!(inp(INTR_STATUS) & INTR_STATUS_DAS0_MASK));  	    	// Poll DAS until conversion is complete

      ain_channel[channel_number].raw.byte[0] = inp(DAS0_LB);     // Read result LB
      ain_channel[channel_number].raw.byte[1] = inp(DAS0_HB);     // Read result HB
    }

    else

    { // Analog inputs [8:15] are on DAS1

      outp(DAS1_CTRL,ain_channel[channel_number].das_ctrl);   		// Start a convesion on the designated channel

      while(!(inp(INTR_STATUS) & INTR_STATUS_DAS1_MASK));  				// Poll DAS until conversion is complete

      ain_channel[channel_number].raw.byte[0] = inp(DAS1_LB);     // Read result LB
      ain_channel[channel_number].raw.byte[1] = inp(DAS1_HB);     // Read result HB
		}

    //--- Calculate the voltage based on settings of BIP and RNG bits
    switch (ain_channel[channel_number].das_ctrl & (DAS_CTRL_BIP_MASK | DAS_CTRL_RNG_MASK))
    {
      case ((!DAS_CTRL_BIP_MASK | !DAS_CTRL_RNG_MASK)):   // 0-5V Range (5V Span)
      {
        ain_channel[channel_number].voltage = ain_channel[channel_number].raw.word * (5.0/4096.0);
        break;
      }

      case ((!DAS_CTRL_BIP_MASK | DAS_CTRL_RNG_MASK)):   // 0-10V Range (10V Span)
      {
        ain_channel[channel_number].voltage = ain_channel[channel_number].raw.word * (10.0/4096.0);
        break;
      }

      case ((DAS_CTRL_BIP_MASK | !DAS_CTRL_RNG_MASK)):   // +/- 5V Range (10V Span)
      {
        ain_channel[channel_number].voltage = ain_channel[channel_number].raw.word * (10.0/4096.0);
        break;
      }

      case ((DAS_CTRL_BIP_MASK | DAS_CTRL_RNG_MASK)):   // +/- 10V Range (20V Span)
      {
        ain_channel[channel_number].voltage = ain_channel[channel_number].raw.word * (20.0/4096.0);
        break;
      }

     default:
        break;
    }

    return(ain_channel[channel_number].raw.word);   						// Return the result

}



/***********************************************************************
    FUNCTION: write_analog_outputs
 DESCRIPTION: Update all the  analog outputs from write values
              pre-written in structure. The channel_number is the only
              channel changed although all channels are updated.

       ENTER: channel_number 	: Is the designated analog output to be changed.
              int_value      	: Is a value between 0x000 (min. FS) and 0xfff (max. FS)

      RETURN: Nothing
***********************************************************************/
void write_analog_output(int channel_number, int int_value)
{

  aout_channel[channel_number].wr_value.word = int_value;

  outp(AOCH0_LB, aout_channel[0].wr_value.byte[0]);  // Preload LB of AOCH0 DAC
  outp(AOCH0_HB, aout_channel[0].wr_value.byte[1]);  // Preload HB of AOCH0 DAC

  outp(AOCH1_LB, aout_channel[1].wr_value.byte[0]);  // Preload LB of AOCH1 DAC
  outp(AOCH1_HB, aout_channel[1].wr_value.byte[1]);  // Preload HB of AOCH1 DAC

  outp(AOCH2_LB, aout_channel[2].wr_value.byte[0]);  // Preload LB of AOCH2 DAC
  outp(AOCH2_HB, aout_channel[2].wr_value.byte[1]);  // Preload HB of AOCH2 DAC

  outp(AOCH3_LB, aout_channel[3].wr_value.byte[0]);  // Preload LB of AOCH3 DAC
  outp(AOCH3_HB, aout_channel[3].wr_value.byte[1]);  // Preload HB of AOCH3 DAC

  outp(AOCH4_LB, aout_channel[4].wr_value.byte[0]);  // Preload LB of AOCH4 DAC
  outp(AOCH4_HB, aout_channel[4].wr_value.byte[1]);  // Preload HB of AOCH4 DAC

  outp(AOCH5_LB, aout_channel[5].wr_value.byte[0]);  // Preload LB of AOCH5 DAC
  outp(AOCH5_HB, aout_channel[5].wr_value.byte[1]);  // Preload HB of AOCH5 DAC

  outp(AOCH6_LB, aout_channel[6].wr_value.byte[0]);  // Preload LB of AOCH6 DAC
  outp(AOCH6_HB, aout_channel[6].wr_value.byte[1]);  // Preload HB of AOCH6 DAC

  outp(AOCH7_LB, aout_channel[7].wr_value.byte[0]);  // Preload LB of AOCH7 DAC
  outp(AOCH7_HB, aout_channel[7].wr_value.byte[1]);  // Preload HB of AOCH7 DAC

  outp(AO_UPDATE, 0);                                // Upadate all DAC channels simultaneously

}


/************************************************************************
    FUNCTION: program_initz
 DESCRIPTION: Intiz the program

       ENTER: Nothing

      RETURN: Nothing
************************************************************************/
void program_initz(void)
{

  //--- Declare local variables
  unsigned int byte_value;

  //--- Setup the user screen
  clrscr();         // Clear the entire user video display screen
  printf("SCIDYNE ADIO-104 Example Program\n");
  printf("Press 'x' to quit program\n");

  //--- Setup ADIO-104 hardware configuration
  byte_value    =
    !CONFIG_PAOVF_IEN_MASK    |  // Bit-7: Pulse Accm Overflow Interrupt enable, 0 = disabled
    !CONFIG_EXT_IEN_MASK      |  // Bit-6: Exteral Interrupt enable, 0 = disabled
    !CONFIG_DAS_IEN_MASK      |  // Bit-5: DAS Interrupt enable, 0 = disabled
    !CONFIG_DAS_CLK_SEL_MASK  |  // Bit-4: DAS clock scaler, 0 = /8, 1 = /4
    !CONFIG_DIR_PB_HI_MASK    |  // Bit-3: PORTB High nibble direction, 0 = Inputs
    !CONFIG_DIR_PB_LO_MASK    |  // Bit-2: PORTB Low nibble direction, 0 = Inputs
    !CONFIG_DIR_PA_HI_MASK    |  // Bit-1: PORTA High nibble direction, 0 = Inputs
    !CONFIG_DIR_PA_LO_MASK;      // Bit-0: PORTA Low nibble direction, 0 = Inputs

	outp(CONFIG,byte_value);   		 // Setup up various ADIO-104 hardware functions

  //--- Initz digital ports
  // Note: Digital ports resort to inputs and their output registers are
  //			 automatically cleared to zero at Power-On-Reset.   This just
  //       assures a known state if program is not started from a hard reset.

  outp(PORTA,0);  	// Set output register bits of PORTA to 0 (low)
	outp(PORTB,0);  	// Set output register bits of PORTB to 0 (low)
	outp(PORTC,0);  	// Set output register bits of PORTC to 0 (low)

	//--- Initz 8-bit Pulse_Accummulator/Counter
  // Note: Pulse Accumulator automatically cleared to zero at Power-On-Reset.
  //   		 This just assures a known state if program is not started from a hard reset.

	outp(PACNT,0);  				                    // Preload the 8-bit pulse accumulator
  outp(INTR_STATUS,INTR_STATUS_PAOVF_MASK);   // Clear any existing PAOVF interrupt


  //--- Clear External Interrupt flip/flop
  // Note: External Interrupt flip/flop is automatically cleared to zero at Power-On-Reset.
  //   This just assures a known state if program is not started from hard reset.

  outp(INTR_STATUS,INTR_STATUS_EXT_MASK);     // Clear any existing EXT interrupt


	//--- Initz Analog Input (DAS) channels 0-15 operating parameters.
  // For simplicty all 16 analog input channels are set for +/-10V Range.
  // However, user can individually change each DAS channel configuration
  // as needed. Bits (6,7) PD0 & PD1 MUST ALWAYS be set to zero for external clock

  //--- DAS AICH0 Located on first MAX197 chip
 	ain_channel[0].raw.word = 0;  	 // Initz raw value to zero
  ain_channel[0].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[0].das_ctrl =    		 // Set DAS AICH0 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 0;                        		   // Bit2-0: Channel address = 0

  //--- DAS AICH1 Located on first MAX197 chip
 	ain_channel[1].raw.word = 0;     // Initz raw value to zero
  ain_channel[1].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[1].das_ctrl =    		 // Set DAS AICH1 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 1;                        		   // Bit2-0: Channel address = 1

  //--- DAS AICH2 Located on first MAX197 chip
 	ain_channel[2].raw.word = 0;  	 // Initz raw value to zero
  ain_channel[2].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[2].das_ctrl =    		 // Set DAS AICH2 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 2;                        		   // Bit2-0: Channel address = 2

  //--- DAS AICH3 Located on first MAX197 chip
 	ain_channel[3].raw.word = 0;     // Initz raw value to zero
  ain_channel[3].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[3].das_ctrl =    		 // Set DAS AICH3 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 3;                        		   // Bit2-0: Channel address = 3

  //--- DAS AICH4 Located on first MAX197 chip
 	ain_channel[4].raw.word = 0;     // Initz raw value to zero
  ain_channel[4].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[4].das_ctrl =    		 // Set DAS AICH4 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 4;                        		   // Bit2-0: Channel address = 4

  //--- DAS AICH5 Located on first MAX197 chip
 	ain_channel[5].raw.word = 0;     // Initz raw value to zero
  ain_channel[5].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[5].das_ctrl =    		 // Set DAS AICH5 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 5;                        		   // Bit2-0: Channel address = 5

  //--- DAS AICH6 Located on first MAX197 chip
 	ain_channel[6].raw.word = 0;     // Initz raw value to zero
  ain_channel[6].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[6].das_ctrl =    		 // Set DAS AICH6 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 6;                        		   // Bit2-0: Channel address = 6

  //--- DAS AICH7 Located on first MAX197 chip
 	ain_channel[7].raw.word = 0;     // Initz raw value to zero
  ain_channel[7].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[7].das_ctrl =    		 // Set DAS AICH7 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 7;                        		   // Bit2-0: Channel address = 7

  //--- DAS AICH8 Located on second MAX197 chip
 	ain_channel[8].raw.word = 0;     // Initz raw value to zero
  ain_channel[8].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[8].das_ctrl =    		 // Set DAS AICH8 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 0;                        		   // Bit2-0: Channel address = 0

  //--- DAS AICH9 Located on second MAX197 chip
 	ain_channel[9].raw.word = 0;     // Initz raw value to zero
  ain_channel[9].voltage  = 0.0;   // Initz voltage to zero
  ain_channel[9].das_ctrl =    		 // Set DAS AICH9 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 1;                        		   // Bit2-0: Channel address = 1

  //--- DAS AICH10 Located on second MAX197 chip
 	ain_channel[10].raw.word = 0;    // Initz raw value to zero
  ain_channel[10].voltage  = 0.0;  // Initz voltage to zero
  ain_channel[10].das_ctrl =    	 // Set DAS AICH10 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 2;                        		   // Bit2-0: Channel address = 2

  //--- DAS AICH11 Located on second MAX197 chip
 	ain_channel[11].raw.word = 0;    // Initz raw value to zero
  ain_channel[11].voltage  = 0.0;  // Initz voltage to zero
  ain_channel[11].das_ctrl =    	 // Set DAS AICH11 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 3;                        		   // Bit2-0: Channel address = 3

  //--- DAS AICH12 Located on second MAX197 chip
 	ain_channel[12].raw.word = 0; 	 // Initz raw value to zero
  ain_channel[12].voltage  = 0.0;  // Initz voltage to zero
  ain_channel[12].das_ctrl =    	 // Set DAS AICH12 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 4;                        		   // Bit2-0: Channel address = 4

  //--- DAS AICH13 Located on second MAX197 chip
 	ain_channel[13].raw.word = 0;    // Initz raw value to zero
  ain_channel[13].voltage  = 0.0;  // Initz voltage to zero
  ain_channel[13].das_ctrl =    	 // Set DAS AICH13 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 5;                        		   // Bit2-0: Channel address = 5

  //--- DAS AICH14 Located on second MAX197 chip
 	ain_channel[14].raw.word = 0;    // Initz voltage to zero
  ain_channel[14].voltage  = 0.0;  // Initz voltage to zero
  ain_channel[14].das_ctrl =    	 // Set DAS AICH14 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 6;                        		   // Bit2-0: Channel address = 6

  //--- DAS AICH15 Located on second MAX197 chip
 	ain_channel[15].raw.word = 0;    // Initz voltage to zero
  ain_channel[15].voltage  = 0.0;  // Initz voltage to zero
  ain_channel[15].das_ctrl =    	 // Set DAS AICH15 parameters
	!DAS_CTRL_PD1_MASK 		|  		   	 // Bit7: PD1 MUST be 0 for external clock
	!DAS_CTRL_PD0_MASK		|  		   	 // Bit6: PD0 MUST be 0 for external clock
	!DAS_CTRL_ACQMOD_MASK |  		     // Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
	 DAS_CTRL_RNG_MASK 		|  		     // Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
	 DAS_CTRL_BIP_MASK    |  		     // Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
 	 7;                        		   // Bit2-0: Channel address = 5

   
	//--- Initz DAC channel 0-7 values
  // Note: Analog Outputs are automatically preset at Power-On-Reset. This sample program
  //	 assumes the ADIO-104 (RM) model is being used.  This step just assures a known
  //	 state if program is not started from a hard reset.
  //   The RM model presets all analog output registers to 0x800  (Mid-Scale)
  //	 The RZ model presets the analog output registers to 0x000  (Zero-Scale)
  
  aout_channel[0].wr_value.word = 0x800;  // Preload output DAC values to mid-scale
  aout_channel[1].wr_value.word = 0x800;  // Preload output DAC values to mid-scale
  aout_channel[2].wr_value.word = 0x800;  // Preload output DAC values to mid-scale
  aout_channel[3].wr_value.word = 0x800;  // Preload output DAC values to mid-scale
  aout_channel[4].wr_value.word = 0x800;  // Preload output DAC values to mid-scale
  aout_channel[5].wr_value.word = 0x800;  // Preload output DAC values to mid-scale
  aout_channel[6].wr_value.word = 0x800;  // Preload output DAC values to mid-scale
  aout_channel[7].wr_value.word = 0x800;  // Preload output DAC values to mid-scale

}


